#include "WRRCU100API.h"

#pragma once

using namespace System;

typedef void (*POSITION_CHANGED_CALLBACK)(INT32 azimuth, INT32 elevation);
typedef void (*STATE_CHANGED_CALLBACK)(UINT32 state);

namespace WRRCU100APIWrapperSpace
{
	//
	// Notification
	//
	public class WRRCU100APIWrapperNotification : public IWRRCU100Notification
	{
		public:

			void __stdcall RegisterCallbacks(POSITION_CHANGED_CALLBACK positionCallback, STATE_CHANGED_CALLBACK stateCallback); // Callback from unmanaged to managed C++

			void __stdcall PositionChanged(IWRRCU100* device, INT32 azimuth, INT32 elevation);
			void __stdcall StateChanged(IWRRCU100* device, UINT32 state);

		private:

			POSITION_CHANGED_CALLBACK positionCallback; // Callback to managed C++
			STATE_CHANGED_CALLBACK stateCallback; // Callback to managed C++
	};


	//
	// Device info
	//
	public ref class WRRCU100APIWrapperDeviceInfo
	{
		public:

			long id;
			String^ serialNumber;
	};


	//
	// Device position, ranges and offsets
	//
	public ref struct WRRCU100APIWrapperPosition
	{
		INT32 azimuth;
		INT32 elevation;
	};

	public ref struct WRRCU100APIWrapperAzimuthRange
	{
		INT32 min;
		INT32 max;
	};

	public ref struct WRRCU100APIWrapperElevationRange
	{
		INT32 min;
		INT32 max;
	};

	public ref struct WRRCU100APIWrapperAzimuthOffsetRange
	{
		INT32 min;
		INT32 max;
	};

	public ref struct WRRCU100APIWrapperElevationOffsetRange
	{
		INT32 min;
		INT32 max;
	};

	public ref struct WRRCU100APIWrapperOffsets
	{
		INT32 azimuth;
		INT32 elevation;
	};


	//
	// Api handler
	//
	public ref class WRRCU100APIWrapper
	{
		public:

			static String^ APILIBRARY = gcnew String("WRRCU100API.dll");
			static const DWORD STATUS_APILIBRARY_LOADED = 100;
			static const DWORD STATUS_APILIBRARY_NOT_FOUND = 200;

			WRRCU100APIWrapper();
			~WRRCU100APIWrapper();

			delegate void PositionChangedCallback(INT32 azimuth, INT32 elevation); // Callback to C#
			delegate void StateChangedCallback(UINT32 state); // Callback to C#

			static void NotificationPositionChangedCallback(INT32 azimuth, INT32 elevation); // Callback for Notification (to unmanaged C++)
			delegate void NotificationPositionChangedDelegate(INT32, INT32); // Callback from Notification (from unmanaged C++)
			static void NotificationStateChangedCallback(UINT32 state); // Callback for Notification (to unmanaged C++)
			delegate void NotificationStateChangedDelegate(UINT32); // Callback from Notification (from unmanaged C++)

			DWORD GetStatus();
			BOOL IsOpened();
			long GetId();
			String^ GetSerialNumber();

			BOOL Enumerate();
			BOOL Open(PositionChangedCallback^ positionCallback, StateChangedCallback^ stateCallback); // C# callbacks
			BOOL Close();
			String^ GetFirmwareVersion();
			UINT32 GetState();
			BOOL StartCalibration();
			void StopCalibration();
			BOOL SetPosition(INT32 azimuth, INT32 elevation);
			WRRCU100APIWrapperPosition^ GetPosition();
			WRRCU100APIWrapperAzimuthRange^ GetAzimuthRange();
			WRRCU100APIWrapperElevationRange^ GetElevationRange();
			BOOL SetOffsets(INT32 azimuth, INT32 elevation);
			WRRCU100APIWrapperOffsets^ GetOffsets();
			WRRCU100APIWrapperAzimuthOffsetRange^ GetAzimuthOffsetRange();
			WRRCU100APIWrapperElevationOffsetRange^ GetElevationOffsetRange();

		private:

			static WRRCU100APIWrapper^ instance;
			DWORD status;

			WRRCU100APIWrapperDeviceInfo deviceInfo;

			PositionChangedCallback^ positionCallback;
			StateChangedCallback^ stateCallback;
	};
}