﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using WRRCU100APIWrapperSpace;

using WRRCU100Example;

namespace WRRCU100Example
{
    public class Wrapper
    {
        public WRRCU100APIWrapper apiWrapper = new WRRCU100APIWrapper();
        public WRRCU100APIWrapperDeviceInfo deviceInfo;
        public String deviceFirmwareVersion;
        public UInt32 deviceState;

        public UInt32 apiStatus;

        public static String APIWRAPPERLIBRARY = WRRCU100APIWrapper.APILIBRARY;
        public static UInt32 APISTATUS_WRAPPERLIBRARY_LOADED = WRRCU100APIWrapper.STATUS_APILIBRARY_LOADED;
        public static UInt32 APISTATUS_WRAPPERLIBRARY_NOTFOUND = WRRCU100APIWrapper.STATUS_APILIBRARY_NOT_FOUND;

        public delegate void DeviceStateChangedHandler(object sender, EventArgs eventArgs);
        public delegate void DevicePositionChangedHandler(object sender, EventArgs eventArgs);
        public event DeviceStateChangedHandler deviceStateChanged;
        public event DevicePositionChangedHandler devicePositionChanged;

        public bool Init()
        {
            apiStatus = apiWrapper.GetStatus();
            if (apiStatus != APISTATUS_WRAPPERLIBRARY_LOADED)
            {
                return false;
            }

            if (apiWrapper.Enumerate() == 0)
            {
                return false;
            }

            WRRCU100APIWrapper.PositionChangedCallback positionCallback = OnDevicePositionChanged;
            WRRCU100APIWrapper.StateChangedCallback stateCallback = OnDeviceStateChanged;
            apiWrapper.Open(positionCallback, stateCallback);

            deviceInfo = new WRRCU100APIWrapperDeviceInfo();
            deviceInfo.id = apiWrapper.GetId();
            deviceInfo.serialNumber = apiWrapper.GetSerialNumber();

            deviceFirmwareVersion = apiWrapper.GetFirmwareVersion();
            deviceState = apiWrapper.GetState();

            return true;
        }

        public UInt32 GetDeviceState()
        {
            return apiWrapper.GetState();
        }

        public Int32 IsDeviceOpened()
        {
            return apiWrapper.IsOpened();
        }

        public void CloseDevice()
        {
            apiWrapper.Close();
        }

        public bool StartDeviceCalibration()
        {
            return apiWrapper.StartCalibration() != 0 ? true : false;
        }

        public void OnDevicePositionChanged(Int32 theAzimuth, Int32 theElevation)
        {
            DeviceEventArgs anEventArgs = new DeviceEventArgs();
            anEventArgs.azimuth = theAzimuth;
            anEventArgs.elevation = theElevation;

            devicePositionChanged(this, anEventArgs);
        }

        public void OnDeviceStateChanged(UInt32 theState)
        {
            DeviceEventArgs anEventArgs = new DeviceEventArgs();
            anEventArgs.state = theState;

            deviceStateChanged(this, anEventArgs);
        }

        public void StopDeviceCalibration()
        {
            apiWrapper.StopCalibration();
        }

        public bool SetDevicePosition(Int32 theAzimuth, Int32 theElevation)
        {
            return apiWrapper.SetPosition(theAzimuth, theElevation) != 0 ? true : false;
        }

        public bool GetDeviceRanges(ref DeviceOffsets thePosition)
        {
            WRRCU100APIWrapperPosition aPosition = apiWrapper.GetPosition();
            if (aPosition != null)
            {
                thePosition.azimuth = aPosition.azimuth;
                thePosition.elevation = aPosition.elevation;

                return true;
            }
            else
            {
                return false;
            }
        }

        public bool GetDeviceRanges(ref DeviceRanges theRanges)
        {
            WRRCU100APIWrapperAzimuthRange anAzimuthRange = apiWrapper.GetAzimuthRange();
            WRRCU100APIWrapperElevationRange anElevationRange = apiWrapper.GetElevationRange();
            if (anAzimuthRange != null && anElevationRange != null)
            {
                theRanges.minAzimuth = anAzimuthRange.min;
                theRanges.maxAzimuth = anAzimuthRange.max;
                theRanges.minElevation = anElevationRange.min;
                theRanges.maxElevation = anElevationRange.max;

                return true;
            }
            else
            {
                return false;
            }
        }

        public bool SetDeviceOffsets(Int32 theAzimuth, Int32 theElevation)
        {
            return apiWrapper.SetOffsets(theAzimuth, theElevation) != 0 ? true : false;
        }

        public bool GetDeviceOffsets(ref DeviceOffsets theOffsets)
        {
            WRRCU100APIWrapperOffsets anOffsets = apiWrapper.GetOffsets();
            if (anOffsets != null)
            {
                theOffsets.azimuth = anOffsets.azimuth;
                theOffsets.elevation = anOffsets.elevation;

                return true;
            }
            else
            {
                return false;
            }
        }

        public bool GetDeviceOffsetRanges(ref DeviceOffsetRanges theOffsetRanges)
        {
            WRRCU100APIWrapperAzimuthOffsetRange anAzimuthOffsetRange = apiWrapper.GetAzimuthOffsetRange();
            WRRCU100APIWrapperElevationOffsetRange anElevationOffsetRange = apiWrapper.GetElevationOffsetRange();
            if (anAzimuthOffsetRange != null && anElevationOffsetRange != null)
            {
                theOffsetRanges.minAzimuthOffset = anAzimuthOffsetRange.min;
                theOffsetRanges.maxAzimuthOffset = anAzimuthOffsetRange.max;
                theOffsetRanges.minElevationOffset = anElevationOffsetRange.min;
                theOffsetRanges.maxElevationOffset = anElevationOffsetRange.max;

                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
