#include <stdio.h>
#include "WRRCU100API.h"

class TNotificationObject:public IWRRCU100Notification
{
public:
    void __stdcall PositionChanged(IWRRCU100 *Device,INT32 Azimuth,INT32 Elevation);
    void __stdcall StateChanged(IWRRCU100 *Device,UINT32 State);
};

HMODULE hAPI;
CREATE_INSTANCE CreateInstance;
IWRRCU100DeviceList *DeviceList;
IWRRCU100 *Device;
WRRCU100_DEVICE_INFO DeviceInfo;
TNotificationObject NotificationObject;
UINT32 DeviceState;


void ShowDeviceState(UINT32 State)
{
    if(State & WRRCU100_STATE_DISCONNECTED)
    {
        printf(" - device is disconnected\n");
    }

    if(State & WRRCU100_STATE_HIGH_TEMPERATURE)
    {
        printf(" - high internal temperature\n");
    }

    if(State & WRRCU100_STATE_LOW_VOLTAGE)
    {
        printf(" - low voltage\n");
    }

    if(State & WRRCU100_STATE_OVERLOAD_AZ)
    {
        printf(" - azimuth motor overload\n");
    }

    if(State & WRRCU100_STATE_OVERLOAD_EL)
    {
        printf(" - elevation motor overload\n");
    }

    if(State & WRRCU100_STATE_AZ_IS_NOT_MOVING)
    {
        printf(" - azimuth is not moving\n");
    }

    if(State & WRRCU100_STATE_EL_IS_NOT_MOVING)
    {
        printf(" - elevation is not moving\n");
    }

    if(State & WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN)
    {
        printf(" - mechanical ranges are unknown\n");
    }

    if(State & WRRCU100_STATE_CALIBRATION_REQUIRED)
    {
        printf(" - calibration is required\n");
    }

    if(State & WRRCU100_STATE_CALIBRATION_IN_PROGRESS)
    {
        printf(" - calibration is in progress\n");
    }
}

void __stdcall TNotificationObject::PositionChanged(IWRRCU100 *Device,INT32 Azimuth,INT32 Elevation)
{
    printf("Position changed: azimuth=%d, elevation=%d.\n",Azimuth,Elevation);

    if(Azimuth==0 && Elevation==0)
    {
        printf("Rotator position is 0,0\n");
        printf("Set position to 50,50\n");
        Device->SetPosition(50,50);
    }
    else if(Azimuth==50 && Elevation==50)
    {
        printf("Rotator position is 50,50\n");
        printf("Done\n");
        PostQuitMessage(0);
    }
}

void __stdcall TNotificationObject::StateChanged(IWRRCU100 *Device,UINT32 State)
{
    printf("Device state changed:\n");
    ShowDeviceState(State);

    if((DeviceState & WRRCU100_STATE_CALIBRATION_IN_PROGRESS) &&
       (State & WRRCU100_STATE_CALIBRATION_IN_PROGRESS)==0)
    {
        //calibration has been finished

        if(State & WRRCU100_STATE_CALIBRATION_REQUIRED)
        {
            //calibration has not been finished successfully, an error occured
            printf("Calibration of the device failed.\n");
            PostQuitMessage(0);
        }
        else
        {
            //calibration has been finished successfully
            printf("Calibration finished successfully.\n");
            printf("Set position to 0,0\n");
            Device->SetPosition(0,0);
        }
    }

    if(State & (~(WRRCU100_STATE_CALIBRATION_IN_PROGRESS |
          WRRCU100_STATE_CALIBRATION_REQUIRED |
          WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN)))
    {
        printf("Device state does not allow to continue.\n");
        PostQuitMessage(0);
    }

    DeviceState=State;
}



int main(int argc, char *argv[])
{
 INT32 Azimuth,Elevation;
 MSG Msg;
 
    printf("WR-RCU-100 CPP example (Microsoft Visual C++ .NET 2003)\n\n");
 
    //load WR-RCU-100 API
    hAPI=LoadLibrary("WRRCU100API.dll");
    
    if(hAPI==NULL)
    {
        printf("Failed to load WRRCU100API.dll.\n");
        return 0;
    }
    
    //get address of exported WR-RCU-100 API function
    CreateInstance=(CREATE_INSTANCE)GetProcAddress(hAPI,"CreateInstance");
     
    //create instance of device list object     
    if(!CreateInstance(WRRCU100_CLASS_ID_DEVICE_LIST,(void**)&DeviceList))
    {
        printf("Failed to create device list object.\n");
        FreeLibrary(hAPI);
        return 0;
    }

    DeviceList->BeginEnumeration();

    //get information about the firt available WR-RCU-100 device
    if(!DeviceList->Next(&DeviceInfo))    
    {
        printf("No available WR-RCU-100 device found.\n");
        DeviceList->EndEnumeration();
        DeviceList->Release();
        FreeLibrary(hAPI);
        return 0;
    }

    //enumerate available WR-RCU-100 devices
    DeviceList->EndEnumeration();

    DeviceList->Release();

    //create instance of device object
    if(!CreateInstance(WRRCU100_CLASS_ID_DEVICE,(void**)&Device))
    {
        printf("Failed to create device object.\n");
        FreeLibrary(hAPI);
        return 0;
    }

    Device->RegisterNotification(&NotificationObject);

    //open the first available WR-RCU-100 device
    if(!Device->Open(DeviceInfo.Id))
    {
        printf("Failed to open device %s.\n",DeviceInfo.SerialNumber);
        Device->UnregisterNotification(&NotificationObject);
        Device->Release();
        FreeLibrary(hAPI);
        return 0;
    }
    
    Device->GetState(&DeviceState);

    printf("Device state:\n");
    ShowDeviceState(DeviceState);

    if(DeviceState & WRRCU100_STATE_CALIBRATION_REQUIRED)
    {
        printf("Calibration of the device is required.\n");
        printf("Starting calibration.\n");
        Device->StartCalibration(FALSE);
    }
    else
    {
        printf("Calibration of the device is not required.\n");

        Device->GetPosition(&Azimuth,&Elevation);

        if(Azimuth==0 && Elevation==0)
        {
            printf("Rotator position in 0,0\n");
            printf("Set position to 50,50\n");
            Device->SetPosition(50,50);
        }
        else
        {
            printf("Set position to 0,0\n");
            Device->SetPosition(0,0);
        }
    }
    
    while(GetMessage(&Msg,0,0,0))
    {
        DispatchMessage(&Msg);
    }

    Device->Close();

    Device->UnregisterNotification(&NotificationObject);

    Device->Release();

    FreeLibrary(hAPI);

	return 0;
}

