unit WRRCU100API;

interface

const
    WRRCU100_CLASS_ID_DEVICE_LIST               = $00000001;
    WRRCU100_CLASS_ID_DEVICE                    = $00000002;

    WRRCU100_STATE_DISCONNECTED                 = $00000001;
    WRRCU100_STATE_HIGH_TEMPERATURE             = $00000002;
    WRRCU100_STATE_LOW_VOLTAGE                  = $00000004;
    WRRCU100_STATE_OVERLOAD_AZ                  = $00000008;
    WRRCU100_STATE_OVERLOAD_EL                  = $00000010;
    WRRCU100_STATE_AZ_IS_NOT_MOVING             = $00000020;
    WRRCU100_STATE_EL_IS_NOT_MOVING             = $00000040;
    WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN    = $00000080;
    WRRCU100_STATE_CALIBRATION_REQUIRED         = $00000100;
    WRRCU100_STATE_CALIBRATION_IN_PROGRESS      = $00000200;

    WRRCU100_LED_GREEN_ON_FLASH_RED_ON          = $00000000;
    WRRCU100_LED_GREEN_ON_RED_ON                = $00000001;
    WRRCU100_LED_GREEN_OFF_RED_ON               = $00000002;
    WRRCU100_LED_GREEN_ON_RED_OFF               = $00000003;
    WRRCU100_LED_GREEN_OFF_RED_OFF              = $00000004;

type

    WRRCU100_DEVICE_INFO = packed record
        SerialNumber:array[0..8] of Char;
        Id:Longint;
    end;

    PWRRCU100_DEVICE_INFO=^WRRCU100_DEVICE_INFO;

    IWRRCU100DeviceList=class
    public
        function GetCount:Longword;virtual;stdcall;abstract;
        function BeginEnumeration:Longbool;virtual;stdcall;abstract;
        function Next(var DeviceInfo:WRRCU100_DEVICE_INFO):Longbool;virtual;stdcall;abstract;
        function EndEnumeration:Longbool;virtual;stdcall;abstract;
        procedure Release;virtual;stdcall;abstract;
    end;

    PWRRCU100DeviceList=^IWRRCU100DeviceList;

    IWRRCU100=class;

    PWRRCU100=^IWRRCU100;

    IWRRCU100Notification=class
        procedure PositionChanged(Device:IWRRCU100;Azimuth,Elevation:Longint);virtual;stdcall;abstract;
        procedure StateChanged(Device:IWRRCU100;State:Longword);virtual;stdcall;abstract;
    end;

    PWRRCU100Notification=^IWRRCU100Notification;

    IWRRCU100=class
        function Open(Id:Longint):Longbool;virtual;stdcall;abstract;
        function Close:Longbool;virtual;stdcall;abstract;
        function IsOpen:Longbool;virtual;stdcall;abstract;
        function GetFirmwareVersion(Buffer:PChar;BufferSize:Longword):Longbool;virtual;stdcall;abstract;
        function GetSerialNumber(Buffer:PChar;BufferSize:Longword):Longbool;virtual;stdcall;abstract;
        function GetState(var State:Longword):Longbool;virtual;stdcall;abstract;
        function GetPosition(var Azimuth,Elevation:Longint):Longbool;virtual;stdcall;abstract;
        function SetPosition(Azimuth,Elevation:Longint):Longbool;virtual;stdcall;abstract;
        function Stop:Longbool;virtual;stdcall;abstract;
        function SetOffsets(AzimuthOffset,ElevationOffset:Longint):Longbool;virtual;stdcall;abstract;
        function GetOffsets(var AzimuthOffset,ElevationOffset:Longint):Longbool;virtual;stdcall;abstract;
        function GetAzimuthOffsetRange(var MinOffset,MaxOffset:Longint):Longbool;virtual;stdcall;abstract;
        function GetElevationOffsetRange(var MinOffset,MaxOffset:Longint):Longbool;virtual;stdcall;abstract;
        function GetAzimuthRange(var MinAzimuth,MaxAzimuth:Longint):Longbool;virtual;stdcall;abstract;
        function GetElevationRange(var MinElevation,MaxElevation:Longint):Longbool;virtual;stdcall;abstract;
        function StartCalibration(ForceToDetermineMechanicalRanges:Longbool):Longbool;virtual;stdcall;abstract;
        procedure StopCalibration;virtual;stdcall;abstract;
        function SetLED(LED:Longword):Longbool;virtual;stdcall;abstract;
        function GetLED(var LED:Longword):Longbool;virtual;stdcall;abstract;
        function RegisterNotification(Notification:IWRRCU100Notification):Longbool;virtual;stdcall;abstract;
        function UnregisterNotification(Notification:IWRRCU100Notification):Longbool;virtual;stdcall;abstract;
        procedure Release;virtual;stdcall;abstract;
    end;

    TCreateInstance=function(ClassId:Longword;out Intf):Longbool;stdcall;

implementation

end.

