program WRRCU100Example;
{$APPTYPE CONSOLE}
uses
  Windows,
  WRRCU100API in 'WRRCU100API.pas';

type
    TNotificationObject=class(IWRRCU100Notification)
    public
        procedure PositionChanged(Device:IWRRCU100;Azimuth,Elevation:Longint);override;stdcall;
        procedure StateChanged(Device:IWRRCU100;State:Longword);override;stdcall;
    end;

var
 hAPI:HMODULE;
 CreateInstance:TCreateInstance;
 DeviceList:IWRRCU100DeviceList;
 Device:IWRRCU100;
 DeviceInfo:WRRCU100_DEVICE_INFO;
 NotificationObject:TNotificationObject;
 DeviceState:Longword;
 Msg:TMsg;
 Azimuth,Elevation:Longint;

procedure ShowDeviceState(State:Longword);
begin
    if (State and WRRCU100_STATE_DISCONNECTED)<>0 then
    begin
        writeln(' - device is disconnected');
    end;

    if (State and WRRCU100_STATE_HIGH_TEMPERATURE)<>0 then
    begin
        writeln(' - high internal temperature');
    end;

    if (State and WRRCU100_STATE_LOW_VOLTAGE)<>0 then
    begin
        writeln(' - low voltage');
    end;

    if (State and WRRCU100_STATE_OVERLOAD_AZ)<>0 then
    begin
        writeln(' - azimuth motor overload');
    end;

    if (State and WRRCU100_STATE_OVERLOAD_EL)<>0 then
    begin
        writeln(' - elevation motor overload');
    end;

    if (State and WRRCU100_STATE_AZ_IS_NOT_MOVING)<>0 then
    begin
        writeln(' - azimuth is not moving');
    end;

    if (State and WRRCU100_STATE_EL_IS_NOT_MOVING)<>0 then
    begin
        writeln(' - elevation is not moving');
    end;

    if (State and WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN)<>0 then
    begin
        writeln(' - mechanical ranges are unknown');
    end;

    if (State and WRRCU100_STATE_CALIBRATION_REQUIRED)<>0 then
    begin
        writeln(' - calibration is required');
    end;

    if (State and WRRCU100_STATE_CALIBRATION_IN_PROGRESS)<>0 then
    begin
        writeln(' - calibration is in progress');
    end;
end;

procedure TNotificationObject.PositionChanged(Device:IWRRCU100;Azimuth,Elevation:Longint);stdcall;
begin
    writeln('Position changed: azimuth=',Azimuth,', elevation=',Elevation,'.');

    if (Azimuth=0) and (Elevation=0) then
        begin
            writeln('Rotator position is 0,0');
            writeln('Set position to 50,50');
            Device.SetPosition(50,50);
        end
    else if (Azimuth=50) and (Elevation=50) then
        begin
            writeln('Rotator position is 50,50');
            writeln('Done');
            PostQuitMessage(0);
        end;
end;

procedure TNotificationObject.StateChanged(Device:IWRRCU100;State:Longword);stdcall;
begin
    writeln('Device state changed:');
    ShowDeviceState(State);

    if ((DeviceState and WRRCU100_STATE_CALIBRATION_IN_PROGRESS)<>0) and
       ((State and WRRCU100_STATE_CALIBRATION_IN_PROGRESS)=0) then
    begin
        //calibration has been finished

        if (State and WRRCU100_STATE_CALIBRATION_REQUIRED)<>0 then
            begin
                //calibration has not been finished successfully, an error occured
                writeln('Calibration of the device failed.');
                PostQuitMessage(0);
            end
        else
            begin
                //calibration has been finished successfully
                writeln('Calibration finished successfully.');
                writeln('Set position to 0,0');
                Device.SetPosition(0,0);
            end;
    end;

    if (State and not (WRRCU100_STATE_CALIBRATION_IN_PROGRESS or
          WRRCU100_STATE_CALIBRATION_REQUIRED or
          WRRCU100_STATE_MECHANICAL_RANGES_UNKNOWN))<>0 then
    begin
        writeln('Device state does not allow to continue.');
        PostQuitMessage(0);
    end;

    DeviceState:=State;
end;

begin
    writeln('WR-RCU-100 PAS example (Delphi 5)');
    writeln;

    Device:=nil;

    //load WR-RCU-100 API
    hAPI:=LoadLibrary('WRRCU100API.dll');

    if hAPI=0 then
    begin
        writeln('Failed to load WRRCU100API.dll.');
        Exit;
    end;

    //get address of exported WR-RCU-100 API function
    CreateInstance:=GetProcAddress(hAPI,'CreateInstance');

    //create instance of device list object
    if not CreateInstance(WRRCU100_CLASS_ID_DEVICE_LIST,DeviceList) then
    begin
        writeln('Failed to create device list object.');
        FreeLibrary(hAPI);
        Exit;
    end;

    //enumerate available WR-RCU-100 devices
    DeviceList.BeginEnumeration;

    //get information about the firt available WR-RCU-100 device
    if not DeviceList.Next(DeviceInfo) then
    begin
        writeln('No available WR-RCU-100 device found.');
        DeviceList.EndEnumeration;
        DeviceList.Release;
        FreeLibrary(hAPI);
        Exit;
    end;

    DeviceList.EndEnumeration;

    DeviceList.Release;

    //create instance of device object
    if not CreateInstance(WRRCU100_CLASS_ID_DEVICE,Device) then
    begin
        writeln('Failed to create device object.');
        FreeLibrary(hAPI);     
        Exit;
    end;

    NotificationObject:=TNotificationObject.Create;

    Device.RegisterNotification(NotificationObject);

    //open the first available WR-RCU-100 device
    if not Device.Open(DeviceInfo.Id) then
    begin
        write('Failed to open device ');
        write(DeviceInfo.SerialNumber);
        writeln('.');
        Device.UnregisterNotification(NotificationObject);
        Device.Release;
        FreeLibrary(hAPI);
        Exit;
    end;

    Device.GetState(DeviceState);

    writeln('Device state:');
    ShowDeviceState(DeviceState);

    if (DeviceState and WRRCU100_STATE_CALIBRATION_REQUIRED)<>0 then
        begin
            writeln('Starting calibration.');
            Device.StartCalibration(FALSE);
        end
    else
        begin
            writeln('Calibration of the device is not required.');

            Device.GetPosition(Azimuth,Elevation);

            if (Azimuth=0) and (Elevation=0) then
                begin
                    writeln('Rotator position in 0,0');
                    writeln('Set position to 50,50');
                    Device.SetPosition(50,50);
                end
            else
                begin
                    writeln('Set position to 0,0');
                    Device.SetPosition(0,0);
                end;
        end;


    while GetMessage(Msg,0,0,0) do
    begin
        DispatchMessage(Msg);
    end;

    Device.Close;

    Device.UnregisterNotification(NotificationObject);

    Device.Release;

    FreeLibrary(hAPI);

    NotificationObject.Destroy;
end.